# -*- coding: utf-8 -*-

from PyQt5.QtWidgets import QMenu, QAction, QMessageBox
from PyQt5.QtGui import QIcon
from qgis.core import QgsApplication, QgsTaskManager, QgsTask, Qgis, QgsMessageLog

from .plugin_fun import *

import os
import win32com.client
import time

# import pydevd_pycharm
# pydevd_pycharm.settrace('localhost', port=44444, stdoutToServer=True, stderrToServer=True)

class Wait(QgsTask):

    def __init__(self, description):
        super().__init__(description, QgsTask.CanCancel)        

    def run(self):
        time.sleep(1)
        return True

    def finished(self, result):
        pass
         
class class_dialog:

    def __init__(self, iface):
        self.iface = iface
        self.pluginPath = os.path.dirname(__file__)

        self.zA_com = None
        self.zB_com = None
        self.cBzA_com = None
        self.cBzB_com = None

    def initGui(self):
        self.mainMenu_geoTime = QMenu(self.iface.mainWindow())
        self.mainMenu_geoTime.setObjectName('GeoTime')
        self.mainMenu_geoTime.setTitle('GeoTime')
        self.menuBar = self.iface.mainWindow().menuBar()
        self.menuBar.insertMenu(self.iface.firstRightStandardMenu().menuAction(), self.mainMenu_geoTime)

        self.toolbar_geoTime = self.iface.addToolBar('GeoTime')

        self.button_zeigeAttribute = QAction('zeige Attribute', self.iface.mainWindow())
        self.button_zeigeAttribute.setIcon(QIcon(os.path.join(self.pluginPath, 'icons', 'ZeigeAttribute.png')))
        self.mainMenu_geoTime.addAction(self.button_zeigeAttribute)
        self.toolbar_geoTime.addAction(self.button_zeigeAttribute)
        self.button_zeigeAttribute.triggered.connect(self.showAttr)

        self.button_uebetrageAusw = QAction('übertrage Auswahl', self.iface.mainWindow())
        self.button_uebetrageAusw.setIcon(QIcon(os.path.join(self.pluginPath, 'icons', 'UebertrageAuswahl.png')))
        self.mainMenu_geoTime.addAction(self.button_uebetrageAusw)
        self.toolbar_geoTime.addAction(self.button_uebetrageAusw)
        self.button_uebetrageAusw.triggered.connect(self.showAttr)

        self.button_holeAusw = QAction('hole Auswahl', self.iface.mainWindow())
        self.button_holeAusw.setIcon(QIcon(os.path.join(self.pluginPath, 'icons', 'HoleAuswahl.png')))
        self.mainMenu_geoTime.addAction(self.button_holeAusw)
        self.toolbar_geoTime.addAction(self.button_holeAusw)
        self.button_holeAusw.triggered.connect(self.showAttr)

        self.button_Zeitbezug = QAction('setze Zeitbezug', self.iface.mainWindow())
        self.button_Zeitbezug.setIcon(QIcon(os.path.join(self.pluginPath, 'icons', 'Zeitbezug.png')))
        self.mainMenu_geoTime.addAction(self.button_Zeitbezug)
        self.toolbar_geoTime.addAction(self.button_Zeitbezug)
        self.button_Zeitbezug.triggered.connect(self.Zeitbezug)
    
    def unload(self):                
        self.mainMenu_geoTime.deleteLater()

    def whenFinishedzB(self):
        if self.cBzB_com.Flag == 0:
            self.startWaitTaskzB()

    def startWaitTaskzB(self):
        ## self.cb_com.Flag can only be assigned by GeoTime if not used by a plugin function
        ## therefore, wait in an separate thread, during which time self.cb_com.Flag is not in use
        ## and after waiting check if the Flag has changed (checking == using)
        self.task = Wait('wait Zeitbezug')
        self.task.taskCompleted.connect(self.whenFinishedzB)
        QgsApplication.taskManager().addTask(self.task)

    def Zeitbezug(self):
        if not self.iface.activeLayer():
            QMessageBox.warning(None, "Fehler", "Kein aktiver Layer gesetzt!")
            return

        ## get db info
        db_info = DB_Info(self.iface).getDBinfo()

        if db_info['Type'] != 'mssql':
            QMessageBox.warning(None, "Fehler", "Aktiver Layer ist keine MSSQL-Layer.")
            return

        if not self.zB_com:
            self.zB_com = win32com.client.Dispatch("GeoTime.GT_Zeitbezug")
        if not self.cBzB_com:
            self.cBzB_com = win32com.client.Dispatch("GT_Callback.Callback")
        self.zB_com.Callback = self.cBzB_com
        self.zB_com.SetzeConnectionParameter4(db_info["User"], db_info["Pwd"], db_info["Server"], db_info["Schema"], db_info["Database"], '')
        self.cBzB_com.Flag = 0
        self.zB_com.SetZeitbezug()

        self.startWaitTaskzB()

    def whenFinishedzA(self):
        if self.cBzA_com.Flag == 0:
            self.startWaitTaskzA()
        else:
            self.iface.activeLayer().setReadOnly(False)

    def startWaitTaskzA(self):
        ## self.cb_com.Flag can only be assigned by GeoTime if not used by a plugin function
        ## therefore, wait in an separate thread, during which time self.cb_com.Flag is not in use
        ## and after waiting check if the Flag has changed (checking == using)
        self.task = Wait('wait zeige Attribute')
        self.task.taskCompleted.connect(self.whenFinishedzA)
        QgsApplication.taskManager().addTask(self.task)

    def showAttr(self):         
        if not self.iface.activeLayer():
            QMessageBox.warning(None,"Fehler", "Kein aktiver Layer gesetzt!")
            return
        
        ## get db info
        db_info = DB_Info(self.iface).getDBinfo()

        if db_info['Type'] != 'mssql':
            QMessageBox.warning(None,"Fehler", "Aktiver Layer ist keine MSSQL-Layer.")
            return
            
        if db_info['featureCount'] == 0:
            QMessageBox.warning(None,"Fehler", "Keine Objekte im aktiven Layer selektiert!")
            return
        else:
            self.zeigeAttribute(db_info)

    
    def zeigeAttribute(self, db_info):

        if not self.zA_com:     
            self.zA_com = win32com.client.Dispatch("GeoTime.GT_ZeigeAttribute")
        if not self.cBzA_com:
            self.cBzA_com = win32com.client.Dispatch("GT_Callback.Callback")
        self.zA_com.Callback = self.cBzA_com
        self.zA_com.UserName = db_info["User"]
        if db_info['Pwd'] != '':
            self.zA_com.Password = db_info['Pwd']
        self.zA_com.Host = db_info['Server']
        self.zA_com.Database = db_info['Database']            
        self.zA_com.Schema = db_info['Schema']
        self.zA_com.FeatureName = db_info['Table']
        if db_info['featureCount'] == 1:
            self.zA_com.NurEingabe = True
        else:
            self.zA_com.NurEingabe = False
        self.zA_com.ResetPrimKeyValues()
        for oid in db_info['featureList']:
            self.zA_com.AddPrimKeyValuesString(str(oid))
        self.cBzA_com.Flag = 0
        self.zA_com.Activate()
        self.iface.activeLayer().setReadOnly(True)
        
        self.startWaitTaskzA()
